@@ -5,11 +5,11 @@
 #
 from .safrs_init import DB, log, SAFRS, dict_merge, test_decorator, SAFRSRequest
 from .errors import ValidationError, GenericError, UnAuthorizedError, NotFoundError
-from .json_encoder import SAFRSJSONEncoder
-from .safrs_api import SAFRSRestAPI, SAFRSAPI
+from .json_encoder import SAFRSJSONEncoder, SAFRSFormattedResponse
+from .safrs_api import SAFRSAPI
 from .base import SAFRSBase
 from .jsonapi_attr import jsonapi_attr
-from .jsonapi import jsonapi_format_response, SAFRSFormattedResponse, paginate
+from .jsonapi import jsonapi_format_response, paginate
 from .api_methods import search, startswith
 from .swagger_doc import jsonapi_rpc
 from .__about__ import __version__, __description__
@@ -20,7 +20,6 @@ __all__ = (
     "__description__",
     #
     "SAFRSAPI",
-    "SAFRSRestAPI",
     # db:
     "SAFRSBase",
     "jsonapi_attr",
