@@ -27,7 +27,7 @@ FILTERABLE = "filterable"
 debug_responses = {
     HTTPStatus.METHOD_NOT_ALLOWED.value: {"description": HTTPStatus.METHOD_NOT_ALLOWED.description},
     HTTPStatus.BAD_REQUEST.value: {"description": HTTPStatus.BAD_REQUEST.description},
-    HTTPStatus.INTERNAL_SERVER_ERROR.value: {"description": HTTPStatus.INTERNAL_SERVER_ERROR.description},
+    HTTPStatus.INTERNAL_SERVER_ERROR.value: {"description": "Internal Server Error"},
 }
 
 # List to generate the swagger references / definitions unique name
@@ -60,7 +60,7 @@ def parse_object_doc(object):
     return api_doc
 
 
-def jsonapi_rpc(http_methods, valid_jsonapi=True):
+def jsonapi_rpc(http_methods=None, valid_jsonapi=True):
     """
     Decorator to expose functions in the REST API:
     When a method is decorated with jsonapi_rpc, this means
@@ -69,7 +69,8 @@ def jsonapi_rpc(http_methods, valid_jsonapi=True):
     :param http_methods:
     :return: function
     """
-
+    if http_methods is None:
+        http_methods = ["POST"]
     def _documented_api_method(method):
         """
         :param method:
@@ -93,7 +94,7 @@ def jsonapi_rpc(http_methods, valid_jsonapi=True):
 
 def is_public(method):
     """
-    :param method:
+    :param method: SAFRSBase method
     :return: True or False, whether the method is to be exposed
     """
 
@@ -102,7 +103,7 @@ def is_public(method):
 
 def get_doc(method):
     """
-    :param  method:
+    :param  method: SAFRSBase method
     :return: OAS documentation
     """
 
@@ -111,8 +112,8 @@ def get_doc(method):
 
 def get_http_methods(method):
     """
-    :param method:
-    :return: a list of http methods used to call this method
+    :param method: SAFRSBase jsonapi_rpc method
+    :return: a list of http methods used to call the method (e.g. POST)
     """
 
     return getattr(method, HTTP_METHODS, ["POST"])
@@ -125,12 +126,19 @@ def SchemaClassFactory(name, properties):
     :param properties: class attributes
     :return: class
     """
-
     # generate a unique name to be used as a reference
     idx = Schema._reference_count.count(name)
     if idx:
+        if Schema._references[name].properties == properties:
+            return Schema._references[name]
         name = name + str(idx)
-    # name = urllib.parse.quote(name)
+
+    for s_name, schema in Schema._references.items():
+        if schema.properties == properties:
+            # this detect duplicate schemas
+            # ignore these for now
+            pass
+
     Schema._reference_count.append(name)
 
     def __init__(self, **kwargs):
@@ -142,6 +150,7 @@ def SchemaClassFactory(name, properties):
             setattr(self, key, value)
 
     new_schema_cls = type(name, (Schema,), {"__init__": __init__, "properties": properties})
+    new_schema_cls.description = ""
     Schema._references[name] = new_schema_cls
     return new_schema_cls
 
@@ -207,6 +216,18 @@ def schema_from_object(name, object):
     return schema
 
 
+def update_response_schema(responses):
+    """
+    Add predefined response schemas if none is available yet
+    """
+    http_codes = {str(status.value): status.description for status in HTTPStatus}
+    for code, response in responses.items():
+        if response and not response.get("schema") and int(code) >= 400:
+            jsonapi_error = {"errors": [{"title": http_codes.get(code, ""), "detail": "", "code": code}]}
+            err_schema = schema_from_object("jsonapi_error_{}".format(code), jsonapi_error)
+            responses[code]["schema"] = err_schema
+
+
 def get_swagger_doc_arguments(cls, method_name, http_method):
     """
     :param cls: class containing the method to be exposed
@@ -227,7 +248,7 @@ def get_swagger_doc_arguments(cls, method_name, http_method):
                    }
     }```
 
-    The schema is created using the values from the documented_api_method decorator,
+    The schema is created using the values from the jsonapi_rpc decorator,
     returned by get_doc()
 
     We use "meta" to remain compliant with the jsonapi schema
@@ -239,7 +260,6 @@ def get_swagger_doc_arguments(cls, method_name, http_method):
     for name, method in inspect.getmembers(cls):
         if name != method_name:
             continue
-
         rest_doc = get_doc(method)
         description = rest_doc.get("description", "")
         if rest_doc:
@@ -272,7 +292,7 @@ def get_swagger_doc_arguments(cls, method_name, http_method):
                             "description": description,
                         }
                         parameters += param
-        else:
+        elif http_method != "options":
             safrs.log.warning('No documentation for method "{}"'.format(method_name))
             # jsonapi_rpc method has no documentation, generate it w/ inspect
             f_args = inspect.getfullargspec(method).args
@@ -340,8 +360,8 @@ def swagger_doc(cls, tags=None):
         responses = {}
         # adhere to open api
         # the model_name will hold the OAS "$ref" schema reference
-        coll_model_name = "{}_{}_coll".format(class_name, http_method)  # collection model name
-        inst_model_name = "{}_{}_inst".format(class_name, http_method)  # instance model name
+        coll_model_name = "{}_coll".format(class_name)  # collection model name
+        inst_model_name = "{}_inst".format(class_name)  # instance model name
 
         sample_dict = cls._s_sample_dict()
 
@@ -355,10 +375,10 @@ def swagger_doc(cls, tags=None):
             sample_instance["relationships"] = sample_rels
 
         coll_sample_data = schema_from_object(coll_model_name, {"data": [sample_instance]})
-        coll_sample_data.description = "Auto generated {}".format(coll_model_name)
+        coll_sample_data.description += "{} {};".format(class_name, http_method)
 
         inst_sample_data = schema_from_object(inst_model_name, {"data": sample_instance})
-        inst_sample_data.description = "Auto generated {}".format(inst_model_name)
+        inst_sample_data.description += "{} {};".format(class_name, http_method)
 
         cls.swagger_models["instance"] = inst_sample_data
         cls.swagger_models["collection"] = coll_sample_data
@@ -388,11 +408,15 @@ def swagger_doc(cls, tags=None):
             # Create the default POST body schema
             sample_dict = cls._s_sample_dict()
             # The POST sample doesn't contain an "id", unless cls.allow_client_generated_ids is True
-            sample_data = schema_from_object(inst_model_name, {"data": {"attributes": sample_dict, "type": cls._s_type}})
             if cls.allow_client_generated_ids:
                 sample_data = schema_from_object(
                     inst_model_name, {"data": {"attributes": sample_dict, "type": cls._s_type, "id": "client_generated"}}
                 )
+            else:
+                sample_data = schema_from_object(inst_model_name, {"data": {"attributes": sample_dict, "type": cls._s_type}})
+
+            sample_data.description += "{} {};".format(class_name, http_method)
+
             parameters.append(
                 {
                     "name": "POST body",
@@ -406,7 +430,7 @@ def swagger_doc(cls, tags=None):
 
         elif http_method == "delete":
             _, responses = cls._s_get_swagger_doc(http_method)
-        else:
+        elif http_method != "options":
             # one of 'options', 'head', 'delete'
             safrs.log.debug('no additional documentation for "{}" '.format(func))
 
@@ -420,6 +444,7 @@ def swagger_doc(cls, tags=None):
         method_doc = parse_object_doc(func)
         safrs.dict_merge(doc, method_doc)
         apply_fstring(doc, locals())
+        update_response_schema(doc["responses"])
         return swagger.doc(doc)(func)
 
     return swagger_doc_gen
@@ -483,6 +508,12 @@ def swagger_relationship_doc(cls, tags=None):
         doc.update(parse_object_doc(func))
 
         responses = {}
+
+        # Shema names (for the swagger "references")
+        model_name = "{}_rel_inst".format(class_name)  # instance model name
+        if cls.relationship.direction in (ONETOMANY, MANYTOMANY):
+            model_name = "{}_rel_coll".format(class_name)  # collection model name
+
         if http_method == "get":
             _, responses = cls._s_get_swagger_doc(http_method)
         elif http_method in ("post", "patch"):
@@ -497,7 +528,8 @@ def swagger_relationship_doc(cls, tags=None):
                 data = [data]
                 responses.pop(HTTPStatus.OK.value, None)
 
-            rel_post_schema = schema_from_object("{}_Relationship".format(class_name), {"data": data})
+            rel_post_schema = schema_from_object(model_name, {"data": data})
+            rel_post_schema.description += "{} {} relationship;".format(class_name, http_method)
             cls.swagger_models["instance"] = rel_post_schema
             cls.swagger_models["collection"] = rel_post_schema
             parameters.append(
@@ -522,7 +554,7 @@ def swagger_relationship_doc(cls, tags=None):
 
             if cls.relationship.direction in (ONETOMANY, MANYTOMANY):
                 data = [data]
-            rel_del_schema = schema_from_object("{}_Relationship".format(class_name), {"data": data})
+            rel_del_schema = schema_from_object(model_name, {"data": data})
             parameters.append(
                 {
                     "name": "{} body".format(class_name),
@@ -533,7 +565,7 @@ def swagger_relationship_doc(cls, tags=None):
                 }
             )
 
-        else:
+        elif http_method != "options":
             # one of 'options', 'head', 'patch'
             safrs.log.info('no documentation for "{}" '.format(http_method))
 
@@ -547,9 +579,11 @@ def swagger_relationship_doc(cls, tags=None):
         doc["responses"] = responses
 
         direction = "to-many" if cls.relationship.direction in (ONETOMANY, MANYTOMANY) else "to-one"
-        parent_name = parent_class.__name__  # to be used by f-string
-        child_name = child_class.__name__  # to be used by f-string
+        parent_name = parent_class.__name__  # referenced by f-string in the jsonapi.py method docstring
+        child_name = child_class.__name__  # referenced by f-string
         apply_fstring(doc, locals())
+        update_response_schema(doc["responses"])
+
         return swagger.doc(doc)(func)
 
     return swagger_doc_gen
